@@ -31,20 +31,22 @@
 rowCollapse <- function(x, idxs, rows = NULL, dim. = dim(x), ...) {
   # Argument 'x':
   if (!is.matrix(x) && !is.vector(x)) {
-    .Deprecated(msg = sprintf("Argument 'x' is of class %s, but should be a matrix or a vector. The use of a %s is not supported, the correctness of the result is not guaranteed, and will be defunct (produce an error) in a future version of matrixStats. Please update your code accordingly.", sQuote(class(x)[1]), sQuote(class(x)[1])))  #nolint
+    .Defunct(msg = sprintf("Argument 'x' is of class %s, but should be a matrix or a vector. The use of a %s is not supported, the correctness of the result is not guaranteed. Please update your code accordingly.", sQuote(class(x)[1]), sQuote(class(x)[1])))  #nolint
   }
 
+  # Argument 'idxs':
+  idxs <- rep(idxs, length.out = dim.[1L])
+
+  # Apply new dimensions
+  if (!identical(dim(x), dim.)) dim(x) <- dim.
+
   # Apply subset
-  if (is.vector(x)) dim(x) <- dim.
   if (!is.null(rows)) {
     x <- x[rows, , drop = FALSE]
     idxs <- idxs[rows]
   }
   dim. <- dim(x)
 
-  # Argument 'idxs':
-  idxs <- rep(idxs, length.out = dim.[1L])
-
   # Columns of interest
   cols <- 0:(dim.[2L] - 1L)
   cols <- cols[idxs]
@@ -61,20 +63,22 @@ rowCollapse <- function(x, idxs, rows = NULL, dim. = dim(x), ...) {
 colCollapse <- function(x, idxs, cols = NULL, dim. = dim(x), ...) {
   # Argument 'x':
   if (!is.matrix(x) && !is.vector(x)) {
-    .Deprecated(msg = sprintf("Argument 'x' is of class %s, but should be a matrix or a vector. The use of a %s is not supported, the correctness of the result is not guaranteed, and will be defunct (produce an error) in a future version of matrixStats. Please update your code accordingly.", sQuote(class(x)[1]), sQuote(class(x)[1])))  #nolint
+    .Defunct(msg = sprintf("Argument 'x' is of class %s, but should be a matrix or a vector. The use of a %s is not supported, the correctness of the result is not guaranteed. Please update your code accordingly.", sQuote(class(x)[1]), sQuote(class(x)[1])))  #nolint
   }
 
+  # Argument 'idxs':
+  idxs <- rep(idxs, length.out = dim.[2L])
+
+  # Apply new dimensions
+  if (!identical(dim(x), dim.)) dim(x) <- dim.
+
   # Apply subset
-  if (is.vector(x)) dim(x) <- dim.
   if (!is.null(cols)) {
     x <- x[, cols, drop = FALSE]
     idxs <- idxs[cols]
   }
   dim. <- dim(x)
 
-  # Argument 'idxs':
-  idxs <- rep(idxs, length.out = dim.[2L])
-
   # Rows of interest
   rows <- seq_len(dim.[1L])
   rows <- rows[idxs]
